package model;

import Jama.SingularValueDecomposition;

/**
 * Třída LSA, obsahující matice U,V obsahující levé a pravé singulární vektory a
 * pole s obsahující singulární hodnoty z matice S
 *
 * @author Matěj Fries
 */
public class SemantickaSVD
{

    /**
     * Matice U a V obsahující levé a pravé singulární vektory
     */
    private final Matice u, v;
    /**
     * Pole singulárních hodnot z diaonální matice S
     */
    private final double[] s;

    /**
     * Konstruktor pro vytvoření sémantického SVD ze třídy
     * SingularValuseDecomposition z knihovny Jama
     *
     * @param svd SingularValueDecpomosition z knihovny Jama
     */
    public SemantickaSVD(SingularValueDecomposition svd)
    {
        s = new double[svd.getSingularValues().length];
        u = new Matice(svd.getU().getArrayCopy());
        v = new Matice(svd.getV().getArrayCopy());
        for (int i = 0; i < svd.getSingularValues().length; i++)
        {
            s[i] = svd.getSingularValues()[i];
        }

    }

    /**
     * Konstruktor pro vytvoření nové sémantické SVD z jiné sSVD
     *
     * @param sem LSA
     */
    public SemantickaSVD(SemantickaSVD sem)
    {
        u = new Matice(sem.getU().getArrayCopy());
        v = new Matice(sem.getV().getArrayCopy());
        s = new double[sem.getShodnoty().length];
        for (int i = 0; i < sem.getShodnoty().length; i++)
        {
            s[i] = sem.getShodnoty()[i];
        }
    }

    /**
     * Konstruktor sémantické SVD z matic a pole hodnot
     *
     * @param u matice U s levými singulárními vektory
     * @param v matice V s pravými singulárními vektory
     * @param hodnotyS pole se singulárními hodnotamy ve formátu double
     */
    public SemantickaSVD(Matice u, Matice v, double[] hodnotyS)
    {
        this.u = new Matice(u.getArrayCopy());
        this.v = new Matice(v.getArrayCopy());
        s = new double[hodnotyS.length];
        for (int i = 0; i < hodnotyS.length; i++)
        {
            s[i] = hodnotyS[i];
        }
    }

    /**
     * Metoda pro snížení počtu témat
     *
     * @param k počet témat
     * @return vrací novou sémantickou SVD se k tématy
     */
    public SemantickaSVD zmensiKoncepty(int k)
    {
        if (k > this.getShodnoty().length || k < 0)
        {
            return new SemantickaSVD(this);
        }
        SemantickaSVD sem;
        double[] sHod = new double[k];
        for (int i = 0; i < k; i++)
        {
            sHod[i] = s[i];
        }
        Matice novaU = zmensiMatici(u, k, false);
        Matice novaV = zmensiMatici(v.transpose(), k, true);
        sem = new SemantickaSVD(novaU, novaV.transpose(), sHod);
        return sem;

    }

    /**
     * privátní metoda pro zmenšení matice na k řádků nebo sloupců
     *
     * @param mat matice ke zmenšení
     * @param k počet konečných řádků nebo sloupců
     * @param radky pokud je true zmenšuje matici o řádky, pakud false zmenšuje
     * matici o sloupce
     * @return vrací novou zmenšenou matici
     */
    private Matice zmensiMatici(Matice mat, int k, boolean radky)
    {
        double[][] hodnoty;
        Matice zmensenaMat;
        if (radky)
        {
            hodnoty = new double[k][mat.getColumnDimension()];
            for (int i = 0; i < k; i++)
            {
                for (int j = 0; j < mat.getColumnDimension(); j++)
                {
                    hodnoty[i][j] = mat.get(i, j);
                }
            }
        } else
        {
            hodnoty = new double[mat.getRowDimension()][k];
            for (int i = 0; i < mat.getRowDimension(); i++)
            {
                for (int j = 0; j < k; j++)
                {
                    hodnoty[i][j] = mat.get(i, j);
                }
            }
        }
        zmensenaMat = new Matice(hodnoty);

        return zmensenaMat;
    }

    /**
     * Vrací singulární hodnoty
     *
     * @return pole singulárních hodnot ve formátu double
     */
    public double[] getShodnoty()
    {
        return s;
    }

    /**
     * Vrací diagonální matici S
     *
     * @return diagonální matice S se singulárními hodnotami
     */
    public Matice getS()
    {
        return Matice.VektorNaDiagonalniMatici(new Vektor(s));
    }

    /**
     * Vrací matici U
     *
     * @return matice U s levými singulárními vektory
     */
    public Matice getU()
    {
        return u;
    }

    /**
     * Vrací matici V
     *
     * @return matice V s pravými singulárními vektory
     */
    public Matice getV()
    {
        return v;
    }

    /**
     * Vrací transponovanou matici V
     *
     * @return transponovaná matice V s pravými singulárními vektory
     */
    public Matice getVT()
    {
        return v.transpose();
    }

    /**
     * Vrací matici S vynásobenou transponovanou maticí V
     *
     * @return vynásobená matice
     */
    public Matice getSkratVT()
    {
        return getS().times(getVT());
    }

    /**
     * Vrací vektory slov z transponované matice V
     *
     * @return pole vektorů
     */
    public Vektor[] getVektoryDokumentu()
    {
        return getSkratVT().getPoleVektoru(Matice.SLOUPCE);
    }
}
